@@ -3,11 +3,12 @@ Author: Olivier Sigaud
 """
 
 import random
+import gym
 
 import numpy as np
 
 from mazemdp.maze_plotter import MazePlotter  # used to plot the maze
-from mazemdp.mdp import Mdp, SimpleActionSpace
+from mazemdp.mdp import Mdp
 from mazemdp.toolbox import E, N, S, W
 
 
@@ -22,7 +23,7 @@ def check_navigability(mdp):
             # Compute the value of state x for each action u of the MDP action space
             if x not in mdp.terminal_states:
                 v_temp = []
-                for u in mdp.action_space.actions:
+                for u in range(mdp.action_space.n):
                     # Process sum of the values of the neighbouring states
                     summ = 0
                     for y in range(mdp.nb_states):
@@ -49,7 +50,7 @@ def build_maze(width, height, walls, hit=False):
 
     # The MDP has one state more than the Maze (the final state
     # outside of the maze)
-    return maze.mdp, maze.nb_states + 1
+    return maze.mdp, maze.nb_states + 1, maze.coord_x, maze.coord_y
 
 
 def create_random_maze(width, height, ratio, hit=False):
@@ -63,9 +64,9 @@ def create_random_maze(width, height, ratio, hit=False):
     while not stop:
         walls = random.sample(range(size), int(n_walls))
 
-        mdp, nb_states = build_maze(width, height, walls, hit=hit)
+        mdp, nb_states, coord_x, coord_y = build_maze(width, height, walls, hit=hit)
         stop = check_navigability(mdp)
-    return mdp, nb_states
+    return mdp, nb_states, coord_x, coord_y
 
 
 class Maze:  # describes a maze-like environment
@@ -75,7 +76,6 @@ class Maze:  # describes a maze-like environment
         height,
         hit=False,
         walls=None,
-        action_list=None,
         nb_actions=4,
         gamma=0.9,
         timeout=50,
@@ -103,32 +103,27 @@ class Maze:  # describes a maze-like environment
         if walls is None:
             walls = []
 
-        if action_list is None:
-            action_list = []
-
         if start_states is None:
             start_states = [0]
 
         self.last_states = last_states
         if self.last_states is None:
             self.last_states = []
-            
+
         self.well = self.nb_states  # all the final states' transitions go there
 
         self.walls = walls
         self.size = width * height
-        state = 0
-        cell = 0
 
         self.state_width = []
         self.state_height = []
+        self.coord_x = np.zeros(height * width - len(walls) + 1)
+        self.coord_y = np.zeros(height * width - len(walls) + 1)
         # ##################### State Space ######################
-        self.init_states(width, height, cell, walls, state)
+        self.init_states(width, height, walls)
 
         # ##################### Action Space ######################
-        self.action_space = SimpleActionSpace(
-            action_list=action_list, nactions=nb_actions
-        )
+        self.action_space = gym.spaces.Discrete(4)
 
         # ##################### Distribution Over Initial States ######################
 
@@ -151,7 +146,7 @@ class Maze:  # describes a maze-like environment
         plotter = MazePlotter(self)  # renders the environment
 
         self.mdp = Mdp(
-            self.nb_states+1,
+            self.nb_states + 1,
             self.action_space,
             start_distribution,
             transition_matrix,
@@ -162,11 +157,15 @@ class Maze:  # describes a maze-like environment
             timeout=timeout,
         )
 
-    def init_states(self, width, height, cell, walls, state):
+    def init_states(self, width, height, walls):
+        state = 0
+        cell = 0
         for i in range(width):
             for j in range(height):
                 if cell not in walls:
                     self.cells[i][j] = state
+                    self.coord_x[state] = j
+                    self.coord_y[state] = i
                     state = state + 1
                     self.state_width.append(i)
                     self.state_height.append(j)
@@ -174,7 +173,7 @@ class Maze:  # describes a maze-like environment
                     self.cells[i][j] = -1
                 cell = cell + 1
 
-        self.nb_states = state
+        assert self.nb_states == state, "maze init: error in the number of states"
 
     def init_transitions(self, hit):
         """
@@ -183,7 +182,7 @@ class Maze:  # describes a maze-like environment
         """
 
         transition_matrix = np.empty(
-            (self.nb_states + 1, self.action_space.size, self.nb_states + 1)
+            (self.nb_states + 1, self.action_space.n, self.nb_states + 1)
         )
 
         transition_matrix[:, N, :] = np.zeros((self.nb_states + 1, self.nb_states + 1))
@@ -232,9 +231,9 @@ class Maze:  # describes a maze-like environment
 
     # --------------------------------- Reward Matrix ---------------------------------
     def simple_reward(self, transition_matrix: np.array):
-        reward_matrix = np.zeros((self.nb_states, self.action_space.size))
+        reward_matrix = np.zeros((self.nb_states, self.action_space.n))
         for from_state, action in zip(*np.nonzero(transition_matrix[:, :, self.well])):
-                reward_matrix[from_state, action] = 1.
+            reward_matrix[from_state, action] = 1.0
         return reward_matrix
 
     # --------------------------------- Reward Matrix ---------------------------------
